package de.intevation.flys.artifacts.model.fixings;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.access.FixAnalysisAccess;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.Parameters;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;

import org.apache.log4j.Logger;


/**
 * Facet to show the W|Q values.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixDeviationFacet
extends      FixingsFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixDeviationFacet.class);

    public static final String [] STD_DEV_COLUMN = { "std-dev" };

    /** Trivial Constructor. */
    public FixDeviationFacet() {
    }


    /**
     * @param name
     */
    public FixDeviationFacet(String name, String description) {
        super(0, name, description, ComputeType.ADVANCE, null, null);
    }

    public FixDeviationFacet(int index, String name, String description) {
        super(index, name, description, ComputeType.ADVANCE, null, null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixDeviationFacet.getData");
        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixAnalysisAccess access = new FixAnalysisAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixAnalysisResult result = (FixAnalysisResult) res.getData();
            double currentKm = getCurrentKm(context);

            Parameters params = result.getParameters();

            double[] stdDev =
                params.interpolate("km", currentKm, STD_DEV_COLUMN);

            if (stdDev == null) {
                logger.warn("getData: stdDev == null at km " + currentKm);
                return null;
            }

            return stdDev;
        }
        else {
            logger.debug("Not an instance of FixationArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixDerivateFacet deepCopy() {
        FixDerivateFacet copy = new FixDerivateFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
